# frozen_string_literal: true

require_relative 'decorators/coin_decorator'

module Abot
  module Info
    class Coin
      include Abot::Info::Coin::CoinDecorator

      attr_reader :base_asset, :quote_asset, :average_price, :current_price, :sell_price, :volume, :num_averaging,
                  :total_quote, :buy_price, :step_averaging, :tick_size, :account, :timer, :trade_params,
                  :pump_detector

      FIAT = %w[USDT BUSD AUD BIDR BRL EUR GBR RUB TRY TUSD USDC DAI IDRT PAX UAH NGN VAI BVND]


      def initialize(options = {})
        @trade_params = options[:trade_params]
        @account = options[:account]
        @base_asset = options[:base_asset]
        @quote_asset = options[:quote_asset]
        @average_price = options[:average_price]
        @current_price = options[:current_price]
        @sell_price = options[:sell_price]
        @volume = options[:volume]
        @num_averaging = options[:num_averaging]
        @total_quote = options[:total_quote]
        @buy_price = options[:buy_price]
        @step_averaging = options[:step_averaging]
        @tick_size = options[:tick_size]
        @timer = options[:timer]
        @pump_detector = options[:pump_detector]
      end

      def self.current_coins(account, trade_params)
        DatabaseTable.data_coins.map do |coin|
          Coin.new(
            trade_params: trade_params,
            account: account,
            base_asset: coin['baseAsset'],
            quote_asset: coin['quoteAsset'],
            average_price: coin['averagePrice'].to_f,
            current_price: coin['askPrice'].to_f,
            sell_price: coin['sellPrice'].to_f,
            volume: coin['allQuantity'].to_f,
            num_averaging: (coin['numAveraging'].to_i - 1),
            total_quote: coin['totalQuote'].to_f.round(2),
            buy_price: coin['buyPrice'].to_f,
            step_averaging: coin['stepAveraging'].to_f,
            tick_size: coin['tickSize'],
            timer: coin['timer'].to_s[0..9],
            pump_detector: coin['pumpDetector'],
          )
        end
      end

      def self.available_coins_number(current_coins, data_settings, free_balance)
        aver_arr = []
        current_coins.each_with_index {|m| aver_arr[m.num_averaging] = ((aver_arr[m.num_averaging].to_i) + 1) }
        aver_weight = 0
        aver_arr.each_with_index { |e, idx| aver_weight += (e.to_i * idx) }
        max_pairs = (free_balance / 100) * data_settings["max_trade_pairs"].to_i
        progressive_max_pairs = max_pairs - aver_weight
        progressive_max_pairs.positive? ? progressive_max_pairs.round(0) : 0
      end

      def self.sum_current_profit(current_coins, quote)
        current_coins = current_coins.select { |s| s.quote_asset == quote }
        current_coins.sum(&:current_profit)
      end

      def self.sum_potential_profit(current_coins, quote)
        current_coins = current_coins.select { |s| s.quote_asset == quote }
        current_coins.sum(&:potential_profit)
      end

      def percent_to_order
        @percent_to_order ||= ((sell_price / current_price - 1) * 100)
      end

      def percent_from_order
        @percent_from_order ||= ((1 - current_price / sell_price) * 100)
      end

      def percent_from_min_to_average
        @percent_to_min ||= ((min_price / next_average_price - 1) * 100)
      end

      def percent_to_max
        @percent_to_max ||= ((sell_price / max_price - 1) * 100)
      end

      def percent_from_max
        @percent_from_max ||= ((1 - max_price / sell_price) * 100)
      end

      def min_price
        @min_price ||= account.symbol_min_price(symbol)
      end

      def max_price
        @max_price ||= account.symbol_max_price(symbol)
      end

      def current_profit
        @current_profit ||= ((current_price - average_price) * volume)
      end

      def potential_profit
        ((sell_price - average_price) * volume)
      end

      def current_quote
        @current_quote ||= (total_quote + current_profit)
      end

      def sell_up
        @sell_up ||= ((potential_profit / total_quote) * 100 - 0.15)
      end

      def next_average_price
        @next_average_price ||=
          (buy_price - (buy_price / 100 * (step_averaging - trade_params['buy_down'].to_f))).round(tick_round_size)
      end

      def next_average_price_percent
        @next_average_price_percent ||= ((current_price / next_average_price - 1) * 100)
      end

      def name
        quote_asset == 'USDT' ? base_asset : symbol
      end

      def symbol
        base_asset + quote_asset
      end

      private

      def tick_round_size
        @tick_round_size ||= tick_size.length - 2
      end

    end
  end
end