# frozen_string_literal: true
require_relative 'helpers'

module Abot
  module Info
    class Table < Terminal::Table
      include Abot::Info::Helpers

      attr_reader :current_coins, :account, :rate_coins, :columns, :trade_params

      ALL_HEADINGS = {
        name: 'Coin',
        name_mobile: 'Coin',
        sell_up: "sell\nup",
        sell_price: 'Ордер',
        max_price: 'Максимум 24h',
        min_price: 'Минимум 24h',
        current_price: 'Тек.цена',
        next_average_price: "Следующее\nусреднение",
        current_quote: "Тек.\nстоим.",
        current_profit: "Тек.\nпрофит",
        potential_profit: "Пот.\nпрофит",
        buy_date: "Дата покупки",
        timer: 'Время',
        current_price_perc_of_order: 'Тек.цена',
        max_price_perc_of_order: 'Максимум 24h',
      }.freeze
      HEADINGS_COINS_TABLE_DEFAULT = %i[
        name
        sell_up
        sell_price
        max_price
        current_price
        min_price
        next_average_price
        current_quote
        buy_date
        timer
        current_profit
        potential_profit
      ].freeze
      HEADINGS_COINS_TABLE_MOBILE = %i[
        name_mobile
        sell_up
        sell_price
        current_price
        next_average_price
      ].freeze
      COLUMN_WITH_TOTAL = %i[
        current_profit
        potential_profit
      ].freeze

      def initialize(options)
        @account = options[:account]
        @columns = options[:columns] || HEADINGS_COINS_TABLE_DEFAULT
        @rate_coins = options.fetch(:rate_coins, [])
        @trade_params ||= DatabaseTable.data_settings
        @current_coins = Coin.current_coins(account, trade_params)


        options = {
          headings: columns.map { |h| { value: ALL_HEADINGS[h], alignment: :center } },
          style: { border_x: '=' }
        }
        super options
      end

      def self.last_columns_set(col)
        col = col.map { |m| ALL_HEADINGS.keys.include?(m) ? m : nil }.compact
        COLUMN_WITH_TOTAL.each { |e| col << col.delete(e) }
        col.compact
      end

      def generate
        sorted_current_coins.each do |coin|
          row = columns.map do |m|
            coin.decorated_cell(m)
          end
          add_row row
        end
        add_separator
        add_row row_total
        self
      end

      def total_averages
        av_arr = []
        current_coins.each { |coin| av_arr[coin.num_averaging] = av_arr[coin.num_averaging].to_i + 1 }
        count = av_arr.compact.sum
        av_arr = av_arr.map.with_index do |av, idx|
          Paint["#{idx}: #{av.to_i}", (Helpers::AVERAGE_COLORS[idx] || :red)] if av.to_i != 0
        end.compact
        av_arr << "∑: #{count}"
        # av_arr << "M: #{available_coins_number}" if available_coins_number
        av_arr.join("\n")
      end

      def info_str
        "#{Helpers.check_coins(account, trade_params["quote_asset"].split(' '))}" \
        "#{Helpers.check_abot}" \
        "#{Helpers.today_info_str}\n" \
        "#{Helpers.balance_str(account, current_coins, trade_params)}\n" \
        "#{Helpers.potential_balance_str(account, current_coins, trade_params)}\n" \
        "#{rate_coins_str}"
      end

      def rate_coins_str
        result = ''
        rate_coins.each_with_index do |c, idx|
          number = colspan / 2
          number += 1 if number.zero?

          result += if idx != 0 && (idx % number).zero?
                      "\n"
                    elsif idx != 0
                      ' '
                    else
                      ''
                    end
          str = Helpers.symbol_price_str(account, c.upcase)
          str.slice!('USDT')
          result += str
        end
        result
      end

      def available_coins_number
        @available_coins_number ||= Coin.available_coins_number(
          current_coins,
          trade_params,
          account.percent_free_balance(current_coins)
        )
      end

      def colspan
        (columns - COLUMN_WITH_TOTAL).count - 1
      end

      def row_total
        row_total = [
          total_averages,
          { value: info_str, colspan: colspan }
        ]
        quote_assets = trade_params['quote_asset'].split(' ')
        cell_current_profit = ''
        cell_potential_profit = ''
        quote_assets.each do |qa|
          if quote_assets.count > 1
            cell_current_profit += "#{Coin.sum_current_profit(current_coins, qa).round(2)} #{qa}\n"if columns.include?(:current_profit)
            cell_potential_profit += "#{Coin.sum_potential_profit(current_coins, qa).round(2)} #{qa}\n" if columns.include?(:potential_profit)
          else
            cell_current_profit += Coin.sum_current_profit(current_coins, qa).round(2).to_s if columns.include?(:current_profit)
            cell_potential_profit += Coin.sum_potential_profit(current_coins, qa).round(2).to_s if columns.include?(:potential_profit)
          end
        end
        row_total << cell_current_profit if columns.include?(:current_profit)
        row_total << cell_potential_profit if columns.include?(:potential_profit)
        row_total
      end

      def sorted_current_coins
        current_coins.sort_by(&:percent_to_order)
      end
    end
  end
end
